import { type FloatingElement, type Middleware, type MiddlewareData, type Placement, type ReferenceElement, type Strategy } from '@floating-ui/dom';
import { type Ref } from 'vue-demi';
import type { MaybeElement, MaybeReadonlyRef } from '../types';
export declare type UseFloatingOptions<T extends ReferenceElement = ReferenceElement> = {
    /**
     * Where to place the floating element relative to its reference element.
     *
     * @default 'bottom'
     */
    placement?: MaybeReadonlyRef<Placement | undefined>;
    /**
     * The type of CSS position property to use.
     *
     * @default 'absolute'
     */
    strategy?: MaybeReadonlyRef<Strategy | undefined>;
    /**
     * These are plain objects that modify the positioning coordinates in some fashion, or provide useful data for the consumer to use.
     *
     * @default undefined
     */
    middleware?: MaybeReadonlyRef<Middleware[] | undefined>;
    /**
     * Callback to handle mounting/unmounting of the elements.
     *
     * @default undefined
     */
    whileElementsMounted?: (reference: T, floating: FloatingElement, update: () => void) => void | (() => void);
};
export declare type UseFloatingReturn = {
    /**
     * The x-coord of the floating element.
     */
    x: Readonly<Ref<number | null>>;
    /**
     * The y-coord of the floating element.
     */
    y: Readonly<Ref<number | null>>;
    /**
     * The stateful placement, which can be different from the initial `placement` passed as options.
     */
    placement: Readonly<Ref<Placement>>;
    /**
     * The type of CSS position property to use.
     */
    strategy: Readonly<Ref<Strategy>>;
    /**
     * Additional data from middleware.
     */
    middlewareData: Readonly<Ref<MiddlewareData>>;
    /**
     * The function to update floating position manually.
     */
    update: () => void;
};
/**
 * Computes the `x` and `y` coordinates that will place the floating element next to a reference element when it is given a certain CSS positioning strategy.
 *
 * @param reference The reference template ref.
 * @param floating The floating template ref.
 * @param options The floating options.
 *
 * @see https://github.com/lozinsky/vue-floating-ui/blob/main/packages/vue-floating-ui/README.md
 */
export declare function useFloating<T extends ReferenceElement = ReferenceElement>(reference: Readonly<Ref<MaybeElement<T>>>, floating: Readonly<Ref<MaybeElement<FloatingElement>>>, options?: UseFloatingOptions<T>): UseFloatingReturn;
